//
//  AntialiasEffect.h
//  AntialiasedRenderTexture
//
//  Created by Piotr on 02/11/16.
//
//

#include "AntialiasEffect.h"

AntialiasEffect* AntialiasEffect::create(Vec2 resolution, float blurRadius, float sampleNum)
{
    AntialiasEffect *pRet = new(std::nothrow) AntialiasEffect();
    if (pRet && pRet->init(resolution, blurRadius, sampleNum))
    {
        pRet->autorelease();
        return pRet;
    }
    else
    {
        delete pRet;
        pRet = nullptr;
        return nullptr;
    }
}

bool AntialiasEffect::init(Vec2 resolution, float blurRadius, float sampleNum) {
    bool success = initGLProgramState("Shaders/antialias_slow.fsh");
    if(success){
        GLfloat gradius = blurRadius;
        GLfloat gsampleNum = sampleNum;
        
        _glprogramstate->setUniformVec2("resolution", resolution);
        _glprogramstate->setUniformFloat("blurRadius", gradius);
        _glprogramstate->setUniformFloat("sampleNum", gsampleNum);
        
        return true;
    }
    return false;
}

/*AntialiasEffect* AntialiasEffect::create(Texture2D* uTexture, float resolution, float radius, Vec2 dir)
{
    AntialiasEffect *pRet = new(std::nothrow) AntialiasEffect();
    if (pRet && pRet->init(uTexture, resolution, radius, dir))
    {
        pRet->autorelease();
        return pRet;
    }
    else
    {
        delete pRet;
        pRet = nullptr;
        return nullptr;
    }
}

bool AntialiasEffect::init(Texture2D* uTexture, float resolution, float radius, Vec2 dir) {
    bool success = initGLProgramState("Shaders/antialias.fsh");
    if(success){
        GLfloat gresolution = resolution;
        GLfloat gradius = radius;
        
        _glprogramstate->setUniformTexture("u_texture", uTexture);
        _glprogramstate->setUniformFloat("resolution", gresolution);
        _glprogramstate->setUniformFloat("radius", gradius);
        _glprogramstate->setUniformVec2("dir", dir);
        
        return true;
    }
    return false;
}*/

bool AntialiasEffect::initGLProgramState(const std::string &fragmentFilename)
{
    auto fileUtiles = FileUtils::getInstance();
    auto fragmentFullPath = fileUtiles->fullPathForFilename(fragmentFilename);
    auto fragSource = fileUtiles->getStringFromFile(fragmentFullPath);
    auto glprogram = GLProgram::createWithByteArrays(ccPositionTextureColor_noMVP_vert, fragSource.c_str());
    
#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID || CC_TARGET_PLATFORM == CC_PLATFORM_WP8 || CC_TARGET_PLATFORM == CC_PLATFORM_WINRT)
    _fragSource = fragSource;
#endif
    
    _glprogramstate = GLProgramState::getOrCreateWithGLProgram(glprogram);
    
    _glprogramstate->retain();
    
    return _glprogramstate != nullptr;
}

AntialiasEffect::AntialiasEffect()
: _glprogramstate(nullptr)
{
#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID || CC_TARGET_PLATFORM == CC_PLATFORM_WP8 || CC_TARGET_PLATFORM == CC_PLATFORM_WINRT)
    _backgroundListener = EventListenerCustom::create(EVENT_RENDERER_RECREATED,
                                                      [this](EventCustom*)
                                                      {
                                                          auto glProgram = _glprogramstate->getGLProgram();
                                                          glProgram->reset();
                                                          glProgram->initWithByteArrays(ccPositionTextureColor_noMVP_vert, _fragSource.c_str());
                                                          glProgram->link();
                                                          glProgram->updateUniforms();
                                                      }
                                                      );
    Director::getInstance()->getEventDispatcher()->addEventListenerWithFixedPriority(_backgroundListener, -1);
#endif
}

AntialiasEffect::~AntialiasEffect()
{
    CC_SAFE_RELEASE_NULL(_glprogramstate);
#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID || CC_TARGET_PLATFORM == CC_PLATFORM_WP8 || CC_TARGET_PLATFORM == CC_PLATFORM_WINRT)
    Director::getInstance()->getEventDispatcher()->removeEventListener(_backgroundListener);
#endif
}
