#include "AppDelegate.h"

AppDelegate::AppDelegate() {
    
}

AppDelegate::~AppDelegate()
{
}

//if you want a different context,just modify the value of glContextAttrs
//it will takes effect on all platforms
void AppDelegate::initGLContextAttrs()
{
    //set OpenGL context attributions,now can only set six attributions:
    //red,green,blue,alpha,depth,stencil
    GLContextAttrs glContextAttrs = {8, 8, 8, 8, 24, 8};
    
    GLView::setGLContextAttrs(glContextAttrs);
}

bool AppDelegate::applicationDidFinishLaunching() {
    // initialize director
    auto director = Director::getInstance();
    auto glview = director->getOpenGLView();
    if(!glview) {
        glview = GLViewImpl::create("Save the Cat");
        director->setOpenGLView(glview);
    }
    
    // turn on display FPS
    director->setDisplayStats(true);
    
    // set FPS. the default value is 1.0/60 if you don't call this
    director->setAnimationInterval(1.0 / 60);
    
    FileUtils::getInstance()->addSearchPath("res");
    FileUtils::getInstance()->addSearchPath("anims");
    string soundsDir = string("sounds_") + SOUNDS_EXT;
    CCLOG("sounds dir: %s", soundsDir.c_str());
    FileUtils::getInstance()->addSearchPath(soundsDir);
    
    NativeHelper::loadSettings();
    NativeHelper::hideBanner();
    if (NativeHelper::isFull()){
        NativeHelper::full = true;
    }
    
    Size size = glview->getFrameSize();
    float scaleFactor = NativeHelper::getDeviceScaleFactor();
    float virtualWidth = 960;
    float virtualHeight = 640;
    float screenWidth = ceilf(size.width * scaleFactor);
    float screenHeight = ceilf(size.height * scaleFactor);
    float virtualRatio = virtualWidth / virtualHeight; //1.5
    float screenRatio = screenWidth / screenHeight;
    float scale = 1;
    
    offsetX = 0;
    offsetY = 0;
    
    //sgs3: 1.777
    float iphone5Width = 1136.0f;
    float iphone5Height = 640.0f;
    float ipadWidth = 1024.0f;
    float ipadHeight = 768.0f;
    float iphone5Ratio = iphone5Width / iphone5Height; //1.775
    float ipadRatio = ipadWidth / ipadHeight; //1.333
    
    if(screenRatio > virtualRatio){ //left and right letterboxing
        scale = virtualRatio / MIN(screenRatio, iphone5Ratio);
        float owidth = iphone5Width;
        offsetX = (1 - scale) * owidth * 0.5;
    }
    else if(screenRatio < virtualRatio){ //up and bottom letterboxing
        scale = screenRatio / MAX(virtualRatio, ipadRatio);
        float oheight = ipadHeight;
        offsetY = (1 - scale) * oheight * 0.5;
    }
    
    virtualWidth = 960;
    virtualHeight = 640;
    
    glview->setDesignResolutionSize(virtualWidth, virtualHeight, ResolutionPolicy::NO_BORDER);
    
    gameScale = scale;
    
    std::srand(unsigned(std::time(0)));
    
    CCLOG("device resolution: %fx%f, scale: %f, offsetX: %f, offsetY: %f", screenWidth, screenHeight, scale, offsetX, offsetY);
    
    // create a scene. it's an autorelease object
    auto scene = MazeScene::createScene();
    
    // run
    director->runWithScene(scene);
    
    return true;
}

// This function will be called when the app is inactive. When comes a phone call,it's be invoked too
void AppDelegate::applicationDidEnterBackground() {
    Director::getInstance()->stopAnimation();
    
    // if you use SimpleAudioEngine, it must be pause
    CocosDenshion::SimpleAudioEngine::getInstance()->pauseBackgroundMusic();
}

// this function will be called when the app is active again
void AppDelegate::applicationWillEnterForeground() {
    Director::getInstance()->startAnimation();
    
    // if you use SimpleAudioEngine, it must resume here
    CocosDenshion::SimpleAudioEngine::getInstance()->resumeBackgroundMusic();
}
