#include "MazeScene.h"

Scene* MazeScene::createScene()
{
    auto scene = Scene::create();
    float gameScale = ((AppDelegate*) Application::getInstance())->gameScale;
    auto layer = MazeScene::create();
    layer->setScale(gameScale);
    scene->addChild(layer);
    return scene;
}

bool MazeScene::init()
{
    if (!Layer::init())
    {
        return false;
    }
    
    float offsetX = ((AppDelegate* )(Application::getInstance()))->offsetX;
    float offsetY = ((AppDelegate* )(Application::getInstance()))->offsetY;
    
    root = CSLoader::createNode("MazeScene.csb");
    
    drawingLayer = root->getChildByName("drawingLayer");
    
    renderTexture = RenderTexture::create(1136, 768);
    renderTexture->setPosition(480, 320);
    renderTexture->beginWithClear(0, 0, 0, 0);
    renderTexture->end();
    
    //super slow
    auto antialiasEffect = AntialiasEffect::create(renderTexture->getSprite()->getTexture()->getContentSizeInPixels(), 2, 2.0f);
    
    //doesn't do anything
    //auto antialiasEffect = AntialiasEffect::create(renderTexture->getSprite()->getTexture(), renderTexture->getSprite()->getTexture()->getContentSizeInPixels().width * renderTexture->getSprite()->getTexture()->getContentSizeInPixels().height, 8, Vec2(1, 0));
    
    
    renderTexture->getSprite()->setGLProgramState(antialiasEffect->getGLProgramState());
    
    drawingLayer->addChild(renderTexture);
    
    drawNode = DrawNode::create();
    drawNode->retain();
    drawNode->setPosition(88, 64);
    
    startNode = root->getChildByName("startNode");
    endNode = root->getChildByName("endNode");
    
    addChild(root);
    
    touchID = -1;
    
    auto listener = EventListenerTouchOneByOne::create();
    listener->setSwallowTouches(true);
    
    listener->onTouchBegan = [&](Touch* touch, Event* event){
        if(touchID == -1){
            Point locationInNode = root->convertToNodeSpace(touch->getLocation());
            
            renderTexture->begin();
            drawNode->clear();
            drawNode->drawDot(locationInNode, DRAW_RADIUS, DRAW_COLOR);
            drawNode->visit();
            renderTexture->end();
            touchID = touch->getID();
            lastLocation = locationInNode;
            
            return true;
        }
        
        return false;
    };
    
    listener->onTouchMoved = [&](Touch* touch, Event* event){
        if(touch->getID() == touchID){
            Point locationInNode = root->convertToNodeSpace(touch->getLocation());
            renderTexture->begin();
            drawNode->clear();
            drawLine(lastLocation.x, lastLocation.y, locationInNode.x, locationInNode.y);
            drawNode->visit();
            renderTexture->end();
            lastLocation = locationInNode;
        }
    };
    
    listener->onTouchEnded = [&](Touch* touch, Event* event){
        if(touch->getID() == touchID){
            touchID = -1;
        }
    };
    
    listener->onTouchCancelled = [&](Touch* touch, Event* event){
        if(touch->getID() == touchID){
            touchID = -1;
        }
    };
    
    _eventDispatcher->addEventListenerWithSceneGraphPriority(listener, this);
    
    return true;
}

void MazeScene::drawLine(int x1, int y1, int x2, int y2){
    int dy = y2 - y1;
    int dx = x2 - x1;
    int stepx, stepy;
    
    if (dy < 0) { dy = -dy;  stepy = -1; } else { stepy = 1; }
    if (dx < 0) { dx = -dx;  stepx = -1; } else { stepx = 1; }
    dy <<= 1;        // dy is now 2*dy
    dx <<= 1;        // dx is now 2*dx
    
    drawNode->drawDot(Vec2(x1, y1), DRAW_RADIUS, DRAW_COLOR);
    
    if (dx > dy)
    {
        int fraction = dy - (dx >> 1);  // same as 2*dy - dx
        while (x1 != x2)
        {
            if (fraction >= 0)
            {
                y1 += stepy;
                fraction -= dx;          // same as fraction -= 2*dx
            }
            x1 += stepx;
            fraction += dy;              // same as fraction -= 2*dy
            drawNode->drawDot(Vec2(x1, y1), DRAW_RADIUS, DRAW_COLOR);
        }
    } else {
        int fraction = dx - (dy >> 1);
        while (y1 != y2) {
            if (fraction >= 0) {
                x1 += stepx;
                fraction -= dy;
            }
            y1 += stepy;
            fraction += dx;
            drawNode->drawDot(Vec2(x1, y1), DRAW_RADIUS, DRAW_COLOR);
        }
    }
}

void MazeScene::onKeyReleased(EventKeyboard::KeyCode keyCode, Event *event){
    if(keyCode == EventKeyboard::KeyCode::KEY_BACK){
        onBackPressed();
    }
}

void MazeScene::onBackPressed(){
    
}

MazeScene::~MazeScene(){
    drawNode->release();
}
