#ifndef  __NATIVE_HELPER_H_
#define  __NATIVE_HELPER_H_

#include <string>
#include <functional>
#include "cocos2d.h"

using namespace std;
USING_NS_CC;

class NativeHelper
{
public:
    //ads
    static void showBanner(int position);
    static void hideBanner();
    static void showInterstitial(string eventName, string adSdk); //eventName for ios only
    static void adsSceneEnter(string name, int bannerPosition); //ios only
    static void adsSceneExit(string name); //ios only
    
    //analytics
    static void reportScreenChange(string name);
    
    //in-app
    static void purchaseProduct(string product);
    static void restorePurchases();
    static void getProductPrice(string product);
    
    //game center
    static void setTotalPoints(int points);
    static void reportScore();
    static void showLeaderboard();
    static void showAchievements();
    static void unlockAchievement(int index);
    
    //utils
    static void sendEmail(string subject, string body, string attachmentPath);
    static void showImagePicker();
    static string getDeviceId();
    static string getAppVersion();
    static string getTempPath();
    static string getStoragePath();
    static void takePhoto();
    static string getRateLink();
    static string getFullLink();
    static bool isFull();
    static void openAppUrl(string url); //ios only
    static float getDeviceScaleFactor();
    static string getBundleId();
    
    //ros
    static void goToVideos();
    
    static void loadSettings();
    static void saveSettings();
    
    //properties
    static bool isBannerShowing;
    static bool full;
    static string fullId;
    
    //callbacks
    std::function<void(vector<string> *products)> purchaseCallback;
    std::function<void()> purchaseCancelledCallback;
    std::function<void(string filename)> imagePickedCallback;
    std::function<void(string filename)> photoTakenCallback;
    std::function<void(string product, string price)> productPriceCallback;
    std::function<void()> videosExitCallback;
    
    //instance required only for setting callback
    static NativeHelper* getInstance();
    
    ~NativeHelper()
    {
        instanceFlag = false;
    }
    
private:
    
    static bool instanceFlag;
    static NativeHelper* instance;
    
    NativeHelper() {
        purchaseCallback = nullptr;
        purchaseCancelledCallback = nullptr;
        imagePickedCallback = nullptr;
        photoTakenCallback = nullptr;
        productPriceCallback = nullptr;
        videosExitCallback = nullptr;
    };
    
};

#endif // __NATIVE_HELPER_H_
