#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)

#include "NativeHelper.h"
#include "platform/android/jni/JniHelper.h"
#include <jni.h>

const char* AppActivityClassName = "org/cocos2dx/cpp/AppActivity";

bool NativeHelper::instanceFlag = false;
bool NativeHelper::isBannerShowing = true;
bool NativeHelper::full = false;
string NativeHelper::fullId = "";
NativeHelper* NativeHelper::instance = NULL;

NativeHelper* NativeHelper::getInstance()
{
    if(!instanceFlag){
        instance = new NativeHelper();
        instanceFlag = true;
    }
    
    return instance;
}

void NativeHelper::showBanner(int position){}

void NativeHelper::hideBanner(){}

void NativeHelper::showInterstitial(string eventName, string adSdk){}

void NativeHelper::reportScreenChange(string name){}

void NativeHelper::purchaseProduct(string product){}

void NativeHelper::restorePurchases(){}

void NativeHelper::getProductPrice(string product){}

string NativeHelper::getDeviceId(){
    cocos2d::JniMethodInfo t;
    if (cocos2d::JniHelper::getStaticMethodInfo(t, AppActivityClassName, "getDeviceId", "()Ljava/lang/String;")){
        
        jstring s = (jstring) t.env->CallStaticObjectMethod(t.classID, t.methodID);
        std::string str = JniHelper::jstring2string(s);
        
        t.env->DeleteLocalRef(t.classID);
        t.env->DeleteLocalRef(s);
        
        return str;
    }
    
    return "";
}

string NativeHelper::getAppVersion(){
    cocos2d::JniMethodInfo t;
    if (cocos2d::JniHelper::getStaticMethodInfo(t, AppActivityClassName, "getAppVersion", "()Ljava/lang/String;")){
        
        jstring s = (jstring) t.env->CallStaticObjectMethod(t.classID, t.methodID);
        std::string str = JniHelper::jstring2string(s);
        
        t.env->DeleteLocalRef(t.classID);
        t.env->DeleteLocalRef(s);
        
        return str;
    }
    
    return "";
}

string NativeHelper::getTempPath(){
    cocos2d::JniMethodInfo t;
    if (cocos2d::JniHelper::getStaticMethodInfo(t, AppActivityClassName, "getTempPath", "()Ljava/lang/String;")){
        
        jstring s = (jstring) t.env->CallStaticObjectMethod(t.classID, t.methodID);
        std::string str = JniHelper::jstring2string(s);
        
        t.env->DeleteLocalRef(t.classID);
        t.env->DeleteLocalRef(s);
        
        return str;
    }
    
    return "";
}

void NativeHelper::sendEmail(string subject, string body, string attachmentPath){
    cocos2d::JniMethodInfo t;
    if (cocos2d::JniHelper::getStaticMethodInfo(t, AppActivityClassName, "sendEmail", "(Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)V")){
        
        jstring stringArg1 = t.env->NewStringUTF(subject.c_str());
        jstring stringArg2 = t.env->NewStringUTF(body.c_str());
        jstring stringArg3 = t.env->NewStringUTF(attachmentPath.c_str());
        t.env->CallStaticVoidMethod(t.classID, t.methodID, stringArg1, stringArg2, stringArg3);
        
        t.env->DeleteLocalRef(t.classID);
        t.env->DeleteLocalRef(stringArg1);
        t.env->DeleteLocalRef(stringArg2);
        t.env->DeleteLocalRef(stringArg3);
    }
}

void NativeHelper::showImagePicker(){}

string NativeHelper::getStoragePath(){}

void NativeHelper::takePhoto(){}

void NativeHelper::adsSceneEnter(string name, int bannerPosition){} //ios only
void NativeHelper::adsSceneExit(string name){} //ios only

string NativeHelper::getRateLink(){
    cocos2d::JniMethodInfo t;
    if (cocos2d::JniHelper::getStaticMethodInfo(t, AppActivityClassName, "getRateLink", "()Ljava/lang/String;")){
        
        jstring s = (jstring) t.env->CallStaticObjectMethod(t.classID, t.methodID);
        std::string str = JniHelper::jstring2string(s);
        
        t.env->DeleteLocalRef(t.classID);
        t.env->DeleteLocalRef(s);
        
        return str;
    }
    
    return "";
}

string NativeHelper::getFullLink(){
    cocos2d::JniMethodInfo t;
    if (cocos2d::JniHelper::getStaticMethodInfo(t, AppActivityClassName, "getFullLink", "()Ljava/lang/String;")){
        
        jstring s = (jstring) t.env->CallStaticObjectMethod(t.classID, t.methodID);
        std::string str = JniHelper::jstring2string(s);
        
        t.env->DeleteLocalRef(t.classID);
        t.env->DeleteLocalRef(s);
        
        return str;
    }
    
    return "";
}

string NativeHelper::getBundleId(){
    cocos2d::JniMethodInfo t;
    if (cocos2d::JniHelper::getStaticMethodInfo(t, AppActivityClassName, "getBundleId", "()Ljava/lang/String;")){
        
        jstring s = (jstring) t.env->CallStaticObjectMethod(t.classID, t.methodID);
        std::string str = JniHelper::jstring2string(s);
        
        t.env->DeleteLocalRef(t.classID);
        t.env->DeleteLocalRef(s);
        
        return str;
    }
    
    return "";
}

bool NativeHelper::isFull(){
    cocos2d::JniMethodInfo t;
    if (cocos2d::JniHelper::getStaticMethodInfo(t, AppActivityClassName, "isFull", "()Z")){
        
        jboolean b = (jboolean) t.env->CallStaticBooleanMethod(t.classID, t.methodID);
        bool result = (bool) b;
        
        t.env->DeleteLocalRef(t.classID);
        
        return result;
    }
    
    return false;
}

void NativeHelper::openAppUrl(string url){} //ios only

void NativeHelper::goToVideos(){}

float NativeHelper::getDeviceScaleFactor(){
    return 1.0f;
}

void NativeHelper::setTotalPoints(int totalPoints){}

void NativeHelper::reportScore(){}

void NativeHelper::showLeaderboard(){}

void NativeHelper::showAchievements(){}

void NativeHelper::unlockAchievement(int index){}

void NativeHelper::loadSettings(){
    NativeHelper::fullId = UserDefault::getInstance()->getStringForKey("fullId", "");
    if(NativeHelper::fullId == NativeHelper::getDeviceId()){
        NativeHelper::full = true;
    }
    else{
        NativeHelper::full = false;
    }
}

void NativeHelper::saveSettings(){
    UserDefault::getInstance()->setStringForKey("fullId", NativeHelper::fullId);
    UserDefault::getInstance()->flush();
}


extern "C"
{
    JNIEXPORT void JNICALL Java_org_cocos2dx_cpp_AppActivity_purchaseCallback(JNIEnv* env, jobject thiz, jobjectArray stringArrays);
    JNIEXPORT void JNICALL Java_org_cocos2dx_cpp_AppActivity_purchaseCancelledCallback(JNIEnv* env, jobject thiz);
    JNIEXPORT void JNICALL Java_org_cocos2dx_cpp_AppActivity_imagePickedCallback(JNIEnv* env, jobject thiz, jstring filename);
    JNIEXPORT void JNICALL Java_org_cocos2dx_cpp_AppActivity_photoTakenCallback(JNIEnv* env, jobject thiz, jstring filename);
    JNIEXPORT void JNICALL Java_org_cocos2dx_cpp_AppActivity_productPriceCallback(JNIEnv* env, jobject thiz, jstring product, jstring price);
    JNIEXPORT void JNICALL Java_org_cocos2dx_cpp_AppActivity_videosCallback(JNIEnv* env, jobject thiz);
};


JNIEXPORT void JNICALL Java_org_cocos2dx_cpp_AppActivity_purchaseCallback(JNIEnv* env, jobject thiz, jobjectArray stringArrays){
    auto helper = NativeHelper::getInstance();
    if(helper->purchaseCallback != nullptr){
        
        vector<string> *products = new vector<string>();
        int size = env->GetArrayLength(stringArrays);
        
        for (int i = 0; i < size; i++){
            jstring javaString = (jstring) env->GetObjectArrayElement(stringArrays, i);
            const char* charString = env->GetStringUTFChars(javaString, 0);
            
            std::string cstring(charString);
            products->push_back(cstring);
            
            env->ReleaseStringUTFChars(javaString, charString);
            env->DeleteLocalRef(javaString);
        }
        
        helper->purchaseCallback(products);
    }
}

JNIEXPORT void JNICALL Java_org_cocos2dx_cpp_AppActivity_purchaseCancelledCallback(JNIEnv* env, jobject thiz){
    auto helper = NativeHelper::getInstance();
    if(helper->purchaseCancelledCallback != nullptr){
        helper->purchaseCancelledCallback();
    }
}

JNIEXPORT void JNICALL Java_org_cocos2dx_cpp_AppActivity_imagePickedCallback(JNIEnv* env, jobject thiz, jstring filename){
    std::string str = JniHelper::jstring2string(filename);
    auto helper = NativeHelper::getInstance();
    if(helper->imagePickedCallback != nullptr){
        helper->imagePickedCallback(str);
    }
}

JNIEXPORT void JNICALL Java_org_cocos2dx_cpp_AppActivity_photoTakenCallback(JNIEnv* env, jobject thiz, jstring filename){
    std::string str = JniHelper::jstring2string(filename);
    auto helper = NativeHelper::getInstance();
    if(helper->photoTakenCallback != nullptr){
        helper->photoTakenCallback(str);
    }
}

JNIEXPORT void JNICALL Java_org_cocos2dx_cpp_AppActivity_productPriceCallback(JNIEnv* env, jobject thiz, jstring product, jstring price){
    std::string _product = JniHelper::jstring2string(product);
    std::string _price = JniHelper::jstring2string(price);
    auto helper = NativeHelper::getInstance();
    if(helper->productPriceCallback != nullptr){
        helper->productPriceCallback(_product, _price);
    }
}

JNIEXPORT void JNICALL Java_org_cocos2dx_cpp_AppActivity_videosCallback(JNIEnv* env, jobject thiz){
    auto helper = NativeHelper::getInstance();
    if(helper->videosExitCallback != nullptr){
        helper->videosExitCallback();
    }
}

#endif
