#include "Utils.h"
#include <errno.h>
#include <ctime>

Sprite* forceLoadSprite(string filename, bool deleteAfterLoad){
    auto futils = FileUtils::getInstance();
    if (FileUtils::getInstance()->isFileExist(filename)){
        
#if (CC_TARGET_PLATFORM == CC_PLATFORM_WP8) || (CC_TARGET_PLATFORM == CC_PLATFORM_WIN32) ||  (CC_TARGET_PLATFORM == CC_PLATFORM_WINRT)
        string slash = "\\";
#else
        string slash = "/";
#endif
        
        string dir = filename.substr(0, filename.rfind(slash) + 1);
        string fname = filename.substr(dir.length());
        
        string prefix = fname.substr(0, fname.rfind("."));
        string extension = fname.substr(fname.rfind("."));
        
        std::time_t result = std::time(nullptr);
        std::ostringstream stream;
        stream << prefix;
        stream << "_" << result;
        stream << extension;
        string randomName = stream.str();
        
        string randomFilePath = dir + randomName;
        
        bool success = futils->renameFile(dir, fname, randomName);
        
        if(success){
            auto sprite = Sprite::create(randomFilePath);
            if(deleteAfterLoad){
                futils->removeFile(randomFilePath);
            }
            else{
                //futils->renameFile(dir, randomName, fname); //rename back
            }
            
            return sprite;
        }
        else{
            CCLOG("loadSpriteRandomName error: %s", strerror(errno));
        }
    }
    else{
        CCLOG("loadSpriteRandomName: File does not exist!");
    }
    
    return nullptr;
}

std::string to_str(int t) {
    std::stringstream ss;
    ss << t;
    std::string str = ss.str();
    return str;
}

std::string to_str(float t) {
    std::stringstream ss;
    ss << t;
    std::string str = ss.str();
    return str;
}

std::string to_str(long t) {
    std::stringstream ss;
    ss << t;
    std::string str = ss.str();
    return str;
}

std::string to_str(double t) {
    std::stringstream ss;
    ss << t;
    std::string str = ss.str();
    return str;
}

int from_str_to_int(std::string t){
    const char* buffer = t.c_str();
    int i = atoi(buffer);
    return i;
}

long from_str_to_long(std::string t){
    const char* buffer = t.c_str();
    long i = atol(buffer);
    return i;
}

double from_str_to_double(std::string t){
    const char* buffer = t.c_str();
    double i = atof(buffer);
    return i;
}

float from_str_to_float(std::string t){
    const char* buffer = t.c_str();
    float i = atof(buffer);
    return i;
}

float f_round(float dval, int n)
{
    char l_fmtp[32], l_buf[64];
    char *p_str;
    sprintf (l_fmtp, "%%.%df", n);
    if (dval>=0)
        sprintf (l_buf, l_fmtp, dval);
    else
        sprintf (l_buf, l_fmtp, dval);
    return ((double)strtod(l_buf, &p_str));
    
}

bool isSpriteTouched(Vec2 pos, Node* child){
    return Rect(0, 0, child->getContentSize().width, child->getContentSize().height).containsPoint(child->convertToNodeSpace(pos));
}

void lookAt(Node* node, cocos2d::Point from, cocos2d::Point to, float radius){
    float deltaX = to.x - from.x;
    float deltaY = to.y - from.y;
    float angle = atan2f(deltaY, deltaX);
    Vec2 pos = Vec2(cosf(angle) * radius + from.x, sinf(angle) * radius + from.y);
    node->setPosition(pos);
}

void playEmitterAt(cocos2d::Point pos, Vector<ParticleSystemQuad* > *memitters, bool burstAtStart, float emissionRate, int index){
    for(int i = 0; i < memitters->size(); i++){
        if((i == index && index != -1) || index == -1){
            auto emitter = memitters->at(i);
            
            emitter->setPosition(pos);
            emitter->setEmissionRate(burstAtStart ? 20 : emissionRate);
            
            emitter->stopAllActions();
            if (burstAtStart){
                emitter->runAction(Sequence::create(DelayTime::create(0.2f), CallFunc::create([emitter, emissionRate]{if (emitter->getEmissionRate() > 0) emitter->setEmissionRate(emissionRate); }), NULL));
            }
        }
    }
}

void playEmitterOnce(cocos2d::Point pos, Vector<ParticleSystemQuad* > *memitters, int amount, float time, int index){
    for(int i = 0; i < memitters->size(); i++){
        if((i == index && index != -1) || index == -1){
            auto emitter = memitters->at(i);
            
            emitter->setPosition(pos);
            emitter->setEmissionRate(amount);
            
            emitter->stopAllActions();
            emitter->runAction(Sequence::create(DelayTime::create(time), CallFunc::create([emitter]{emitter->setEmissionRate(0); }), NULL));
        }
    }
}

void stopEmitters(Vector<ParticleSystemQuad*> *memitters){
    for(int i = 0; i < memitters->size(); i++){
        auto emitter = memitters->at(i);
        emitter->stopAllActions();
        emitter->setEmissionRate(0);
    }
}

ParticleSystemQuad* createEmitter(string filename){
    auto emitter = ParticleSystemQuad::createWithTotalParticles(100);
    auto director = Director::getInstance();
    director->getTextureCache()->addImage(filename);
    emitter->setTexture(Director::getInstance()->getTextureCache()->getTextureForKey(filename.c_str()));
    emitter->setDuration(-1);
    emitter->setAngle(0);
    emitter->setAngleVar(180);
    emitter->setSpeed(150);
    emitter->setSpeedVar(50);
    emitter->setPosVar(Point(0, 0));
    emitter->setPosition(Point(0, 0));
    emitter->setLife(4);
    emitter->setLifeVar(0);
    emitter->setStartSpin(0);
    emitter->setStartSpinVar(0);
    emitter->setEndSpin(1440);
    emitter->setEndSpinVar(0);
    emitter->setStartColor(Color4F(1, 1, 1, 1));
    emitter->setStartColorVar(Color4F(0, 0, 0, 0));
    emitter->setEndColor(Color4F(1, 1, 1, 1));
    emitter->setEndColorVar(Color4F(0, 0, 0, 0));
    emitter->setStartSize(30);
    emitter->setStartSizeVar(0);
    emitter->setEndSize(0);
    emitter->setEndSizeVar(0);
    emitter->setEmissionRate(0);
    
    return emitter;
}

void uPlayBackgroundMusic(string filenameWithoutExtension){
    string extension = SOUNDS_EXT;
#if (CC_TARGET_PLATFORM == CC_PLATFORM_IOS || CC_TARGET_PLATFORM == CC_PLATFORM_MAC)
    string tfilename = filenameWithoutExtension + ".caf";
    if(FileUtils::getInstance()->isFileExist(tfilename)){
        extension = "caf";
    }
#endif
    
    string filename = filenameWithoutExtension + "." + extension;
    
    CocosDenshion::SimpleAudioEngine::getInstance()->playBackgroundMusic(filename.c_str());
}

void uPlayBackgroundMusic(string filenameWithoutExtension, bool loop){
    bool musicOn = UserDefault::getInstance()->getBoolForKey("musicOn", true);
    if(!musicOn) return;
    string extension = SOUNDS_EXT;
#if (CC_TARGET_PLATFORM == CC_PLATFORM_IOS || CC_TARGET_PLATFORM == CC_PLATFORM_MAC)
    string tfilename = filenameWithoutExtension + ".caf";
    if(FileUtils::getInstance()->isFileExist(tfilename)){
        extension = "caf";
    }
#endif
    
    string filename = filenameWithoutExtension + "." + extension;
    
    CocosDenshion::SimpleAudioEngine::getInstance()->playBackgroundMusic(filename.c_str(), loop);
}

void uStopBackgroundMusic(){
    CocosDenshion::SimpleAudioEngine::getInstance()->stopBackgroundMusic();
}

void uPauseEffect(unsigned int soundId){
    CocosDenshion::SimpleAudioEngine::getInstance()->pauseEffect(soundId);
}

void uStopEffect(unsigned int soundId){
    CocosDenshion::SimpleAudioEngine::getInstance()->stopEffect(soundId);
}

void uResumeEffect(unsigned int soundId){
    CocosDenshion::SimpleAudioEngine::getInstance()->resumeEffect(soundId);
}

unsigned int uPlayEffect(string filenameWithoutExtension, bool loop){
    string extension = SOUNDS_EXT;
#if (CC_TARGET_PLATFORM == CC_PLATFORM_IOS || CC_TARGET_PLATFORM == CC_PLATFORM_MAC)
    string tfilename = filenameWithoutExtension + ".caf";
    if(FileUtils::getInstance()->isFileExist(tfilename)){
        extension = "caf";
    }
#endif
    
    string filename = filenameWithoutExtension + "." + extension;
    
    return CocosDenshion::SimpleAudioEngine::getInstance()->playEffect(filename.c_str(), loop);
}

unsigned int uPlayEffect(string filenameWithoutExtension){
    
    string extension = SOUNDS_EXT;
#if (CC_TARGET_PLATFORM == CC_PLATFORM_IOS || CC_TARGET_PLATFORM == CC_PLATFORM_MAC)
    string tfilename = filenameWithoutExtension + ".caf";
    if(FileUtils::getInstance()->isFileExist(tfilename)){
        extension = "caf";
    }
#endif
    
    string filename = filenameWithoutExtension + "." + extension;
    
    return CocosDenshion::SimpleAudioEngine::getInstance()->playEffect(filename.c_str());
}

void uPreloadEffect(string filenameWithoutExtension){
    
    string extension = SOUNDS_EXT;
#if (CC_TARGET_PLATFORM == CC_PLATFORM_IOS || CC_TARGET_PLATFORM == CC_PLATFORM_MAC)
    string tfilename = filenameWithoutExtension + ".caf";
    if(FileUtils::getInstance()->isFileExist(tfilename)){
        extension = "caf";
    }
#endif
    
    string filename = filenameWithoutExtension + "." + extension;
    
    CocosDenshion::SimpleAudioEngine::getInstance()->preloadEffect(filename.c_str());
}

void uUnloadEffect(string filenameWithoutExtension){
    
    string extension = SOUNDS_EXT;
#if (CC_TARGET_PLATFORM == CC_PLATFORM_IOS || CC_TARGET_PLATFORM == CC_PLATFORM_MAC)
    string tfilename = filenameWithoutExtension + ".caf";
    if(FileUtils::getInstance()->isFileExist(tfilename)){
        extension = "caf";
    }
#endif
    
    string filename = filenameWithoutExtension + "." + extension;
    
    CocosDenshion::SimpleAudioEngine::getInstance()->unloadEffect(filename.c_str());
}

bool uIsBackgroundMusicPlaying(){
    return CocosDenshion::SimpleAudioEngine::getInstance()->isBackgroundMusicPlaying();
}

void translateNode(Node* node, Vec2 vec){
    node->setPosition(node->getPosition() + vec);
}

void fillWithFrames(Vector<SpriteFrame* > *v, string name, int from, int to, int back, int repeat){
    if(from == to) return;
    int d = from < to ? 1 : -1;
    for(int r = 0; r < repeat; r++){
        
        int i = from;
        while(i != to){
            addFrame(v, name, i);
            i += d;
        }
        if(to == back){
            addFrame(v, name, to);
            continue;
        }
        while(i != back){
            addFrame(v, name, i);
            i -= d;
        }
        addFrame(v, name, back);
    }
}

void addFrame(Vector<SpriteFrame* > *v, string name, int index){
    char str[100] = {0};
    sprintf(str, name.c_str(), index);
    auto sprite = Sprite::create(str);
    //    CCLOG("%s", str);
    //    CCLOG("%d", index);
    v->pushBack(sprite->getSpriteFrame());
}

void setMultipleMixes(SkeletonAnimation* anim, vector<string> names1, vector<string> names2, bool both, float time){
    for(int j = 0; j < names2.size(); j++){
        for(int k = 0; k < names1.size(); k++){
            string n1 = names1.at(k);
            string n2 = names2.at(j);
            anim->setMix(n1, n2, time);
            if(both) anim->setMix(n2, n1, time);
        }
    }
}

void setAllMixes(SkeletonAnimation* anim, float time){
    vector<string> animations;
    for(int i = 0; i < anim->getState()->data->skeletonData->animationsCount; i++){
        auto a = anim->getState()->data->skeletonData->animations[i];
        string animation = a->name;
        animations.push_back(animation);
    }
    setMultipleMixes(anim, animations, animations, true, time);
}

string formatTimeFloat(float number){
    char s[25];
    sprintf(s, "%.2f", number);
    return s;
}

string formatTimeFloat1(float number){
    char s[25];
    sprintf(s, "%.1f", number);
    return s;
}

string getCurrentDateAndTime(time_t offset){
    time_t rawtime;
    struct tm * timeinfo;
    char buffer[80];
    
    time (&rawtime);
    rawtime += offset;
    timeinfo = localtime(&rawtime);
    
    strftime(buffer,80,"%d-%m-%Y %H:%M:%S",timeinfo);
    std::string str(buffer);
    
    return str;
}

time_t getCurrentDateAndTimeAsLong(time_t offset){
    time_t rawtime;
    struct tm * timeinfo;
    
    time (&rawtime);
    rawtime += offset;
    timeinfo = localtime(&rawtime);
    
    time_t t = mktime(timeinfo);
    return t;
}

void split(const std::string &s, char delim, std::vector<std::string> &elems) {
    std::stringstream ss;
    ss.str(s);
    std::string item;
    while (std::getline(ss, item, delim)) {
        elems.push_back(item);
    }
}


std::vector<std::string> split(const std::string &s, char delim) {
    std::vector<std::string> elems;
    split(s, delim, elems);
    return elems;
}
