/****************************************************************************
Copyright (c) 2008-2010 Ricardo Quesada
Copyright (c) 2010-2012 cocos2d-x.org
Copyright (c) 2011      Zynga Inc.
Copyright (c) 2013-2014 Chukong Technologies Inc.

http://www.cocos2d-x.org

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
****************************************************************************/
package org.cocos2dx.cpp;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigInteger;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.util.ArrayList;
import java.util.HashMap;

import org.cocos2dx.lib.Cocos2dxActivity;
import org.cocos2dx.lib.Cocos2dxGLSurfaceView;
import org.cocos2dx.lib.Cocos2dxHelper;
import org.json.JSONObject;

import com.google.android.gms.ads.AdListener;
import com.google.android.gms.ads.AdRequest;
import com.google.android.gms.ads.AdSize;
import com.google.android.gms.ads.AdView;
import com.google.android.gms.ads.InterstitialAd;
import com.google.android.gms.analytics.GoogleAnalytics;
import com.google.android.gms.analytics.Tracker;
import com.google.android.gms.auth.GoogleAuthException;
import com.google.android.gms.auth.GoogleAuthUtil;
import com.google.android.gms.auth.UserRecoverableAuthException;
import com.google.android.gms.analytics.Logger.LogLevel;
import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.GooglePlayServicesUtil;
import com.google.android.gms.common.Scopes;
import com.google.android.gms.common.api.GoogleApiClient;
import com.proliberis.Pszczoly.R;

import android.annotation.TargetApi;
import android.app.Activity;
import android.app.Dialog;
import android.app.PendingIntent;
import android.content.ComponentName;
import android.content.Context;
import android.content.CursorLoader;
import android.content.Intent;
import android.content.IntentSender;
import android.content.ServiceConnection;
import android.content.pm.ActivityInfo;
import android.content.pm.PackageManager.NameNotFoundException;
import android.database.Cursor;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Bundle;
import android.os.Environment;
import android.os.IBinder;
import android.provider.MediaStore;
import android.provider.Settings.Secure;
import android.util.Log;
import android.view.View;
import android.widget.RelativeLayout;
import android.widget.RelativeLayout.LayoutParams;

public class AppActivity extends Cocos2dxActivity {

	private static final String TAG = AppActivity.class.getSimpleName();

	private static AppActivity _appActivity;

	public static native void purchaseCallback(String[] products);
	public static native void purchaseCancelledCallback();
	public static native void imagePickedCallback(String filename);
	public static native void photoTakenCallback(String filename);
	public static native void productPriceCallback(String product, String price);
	public static native void videosCallback();
	public static native void firebaseLoggedInCallback(String result);
	public static native void firebaseLoggedOutCallback();
	public static native void firebaseDataCallback(String path, String data);

	private Cocos2dxGLSurfaceView glSurfaceView;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		_appActivity = this;

		this.setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
	}

	public static void showBanner(int position) {}

	public static void hideBanner() {}

	public static void showInterstitial(final String adSdk){}

	public static void reportScreenChange(final String name) {}

	public static String getDeviceId(){
		//https://www.random.org/strings/?num=2&len=10&digits=on&upperalpha=on&loweralpha=on&unique=on&format=html&rnd=new
		String salt0 = "eWpqI3neIC";
		String salt1 = "ojZWY9mUXi";
		String secureString = Secure.getString(_appActivity.getContentResolver(), Secure.ANDROID_ID);
		String stringToEncrypt = salt0 + secureString + salt1;
		MessageDigest messageDigest;
		try {
			messageDigest = MessageDigest.getInstance("SHA-256");
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
			return stringToEncrypt;
		}
		messageDigest.update(stringToEncrypt.getBytes());
		String encryptedString = new String(toHex(messageDigest.digest()));

		return encryptedString;
	}

	private static String toHex(byte[] array)
    {
        BigInteger bi = new BigInteger(1, array);
        String hex = bi.toString(16);
        int paddingLength = (array.length * 2) - hex.length();
        if(paddingLength > 0)
            return String.format("%0" + paddingLength + "d", 0) + hex;
        else
            return hex;
    }

	public static String getAppVersion(){
        try {
			return _appActivity.getPackageManager().getPackageInfo(_appActivity.getPackageName(), 0).versionName;
		}
        catch (NameNotFoundException e) {
        	return "";
        }
	}

	public static void purchaseProduct(final String product){}

	public static void restorePurchases(){}

	public static void sendEmail(final String subject, final String body, final String attachmentPath){
		_appActivity.runOnUiThread(new Runnable() {

			@Override
			public void run() {

				File f = new File(attachmentPath);
				Log.d("MYFILE", f.getAbsolutePath() + ", exists: " + f.exists());
				Intent sendIntent;

				sendIntent = new Intent(Intent.ACTION_SEND);
				sendIntent.setType("message/rfc822");
				sendIntent.putExtra(Intent.EXTRA_SUBJECT, subject);
				sendIntent.putExtra(Intent.EXTRA_EMAIL, new String[]{"contact@123kidsfun.com"});
				sendIntent.putExtra(Intent.EXTRA_TEXT, body);
				if(f.exists()){
					sendIntent.putExtra(Intent.EXTRA_STREAM, Uri.fromFile(f));
					sendIntent.setType("image/png");
					Log.d("EMAIL", "attachment " + attachmentPath + " added");
				}
				else{
					Log.d("EMAIL", "file " + attachmentPath + " does not exist!");
				}

				Log.d("EMAIL", "subject = " + subject + ", body = " + body);

				_appActivity.startActivity(Intent.createChooser(sendIntent, "Send Mail"));
			}

		});
	}

	public static void showImagePicker(){}

	public static String getTempPath(){
		String path = Environment.getExternalStorageDirectory().getAbsolutePath() + "/.rosmedia/";
		File f = new File(path);
		f.mkdirs();
		Log.d("getTempPath", "path = " + path);
		return path;
	}

	public static String getStoragePath(){
		String path = _appActivity.getApplicationInfo().dataDir + "/";
		Log.d("getStoragePath", "path = " + path);
		return path;
	}

	public static void takePhoto(){}

	public static String getRateLink(){
		String rateLink = "market://details?id=" + _appActivity.getPackageName();
		return rateLink;
	}

	public static String getFullLink(){
		String rateLink = "market://details?id=" + _appActivity.getPackageName() + "Full";
		return rateLink;
	}

	public static void setTotalPoints(final int points){}

	public static void reportScore(){}

	public static void showLeaderboard(){}

	public static void showAchievements(){}

	public static void unlockAchievement(final int index){}

	public static String getBundleId(){
		return _appActivity.getPackageName();
	}

	public static boolean isFull(){
		return false;
	}

	public static void goToVideos(){}

	@Override
	protected void onResume() {
		super.onResume();
		glSurfaceView.setOnSystemUiVisibilityChangeListener (new View.OnSystemUiVisibilityChangeListener() {
	        @Override
	        public void onSystemUiVisibilityChange(int visibility) {
	            if ((visibility & View.SYSTEM_UI_FLAG_FULLSCREEN) == 0) {
	            	hideSystemUI();
	            }
	        }
	    });
	}

	@Override
	protected void onPause() {
		super.onPause();
	}

	@Override
	protected void onDestroy() {
		super.onDestroy();
	}

	@Override
	public void onBackPressed() {
	    super.onBackPressed();
	}

	@Override
	public void onStart() {
	    super.onStart();
	}

	@Override
	public void onStop() {
		super.onStop();
	}

    public Cocos2dxGLSurfaceView onCreateView()
    {
        glSurfaceView = new Cocos2dxGLSurfaceView(this);
        this.hideSystemUI();
        glSurfaceView.setEGLConfigChooser(5, 6, 5, 0, 16, 8);

        return glSurfaceView;
    }

    public void onWindowFocusChanged(boolean hasFocus)
    {
        super.onWindowFocusChanged(hasFocus);
        if (hasFocus)
        {
            this.hideSystemUI();
        }
    }

    @TargetApi(Build.VERSION_CODES.KITKAT)
    private void hideSystemUI()
    {
        if (Build.VERSION.SDK_INT >= 19) {
	        glSurfaceView.setSystemUiVisibility(
	                Cocos2dxGLSurfaceView.SYSTEM_UI_FLAG_LAYOUT_STABLE
	                | Cocos2dxGLSurfaceView.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION
	                | Cocos2dxGLSurfaceView.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
	                | Cocos2dxGLSurfaceView.SYSTEM_UI_FLAG_HIDE_NAVIGATION
	                | Cocos2dxGLSurfaceView.SYSTEM_UI_FLAG_FULLSCREEN
	                | Cocos2dxGLSurfaceView.SYSTEM_UI_FLAG_IMMERSIVE_STICKY);
	    }
    }
}
